--Author: Brain 
-- All right reserved

Volokusha = {};

function Volokusha.prerequisitesPresent(specializations)
   return true;
end;

function Volokusha:load(xmlFile)
	self.vehicleType = getXMLString(xmlFile, "vehicle#type");
	self.OpenV = false;
	local rotLeftNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.rotLeft#index"));
    if rotLeftNode ~= nil then
        self.rotLeft = {};
        self.rotLeft.node = rotLeftNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotLeft#minRot"));
        self.rotLeft.minRot = {};
        self.rotLeft.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotLeft.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotLeft.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.rotLeft#maxRot"));
        self.rotLeft.maxRot = {};
        self.rotLeft.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotLeft.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotLeft.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
        self.rotLeft.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotLeft#rotTime"), 2)*1000;
		self.rotLeft.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.rotLeft#touchRotLimit"), 10));
    end;

	-- animation
	self.anim = SpecializationUtil.callSpecializationsFunction("anim");
	self.Go = {};
	self.Done = {};
	self.charId = {};
	self.clipIndex = {};
	self.CheckDone = {};
	self.animParts = {}
	self.Activated = {};

	self.startpoint = {};
	self.loopCheck = {};
	self.Speed = {};


	local count = getXMLInt(xmlFile, "vehicle.animParts#count");
	local part = self.animParts;
    for i=1, count do
		part[i] = {};
		local partname = string.format("vehicle.animParts.part".."%d", i);
	    local nameR = getXMLString(xmlFile, partname.."#name");
		self.charId[nameR] = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#rootNode"));
		self.clipIndex[nameR] = getXMLString(xmlFile, partname.."#animationClip");
		self.CheckDone[nameR] = false;
		self.Activated[nameR] = false;

		self.startpoint[nameR] = Utils.getNoNil(getXMLFloat(xmlFile, partname.."#startpoint"),0.0);
		local charId = getAnimCharacterSet(self.charId[nameR]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[nameR]);
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackTime(charId, 0, self.startpoint[nameR]);
		enableAnimTrack(charId, 0);
		disableAnimTrack(charId, 0);
		self.loopCheck[nameR] = false;
		self.Speed[nameR] = 1.0;
	end;

end;

function Volokusha:delete()
end;

function Volokusha:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Volokusha:keyEvent(unicode, sym, modifier, isDown)
end;

function Volokusha:update(dt)

	if self.Go.animFoldExtend ~= nil and self.Done.animFoldExtend ~= false then
		self:anim("animFoldExtend", false);
	end;

	if self.attacherVehicle then
		if self.fixAttacher then
			local implement = self.attacherVehicle:getImplementByObject(self);
			if implement ~= nil then
				local joint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.saveJoint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.maxRotJoint = joint.maxRotLimit;
				self.attacherJointIndex = implement.jointDescIndex;
				self.rotationCopy  = joint.minRot;
				joint.minRot = {Utils.degToRad(2),0,0};
				joint.maxRotLimit = {0, 0, 0};
				setJointRotationLimit(joint.jointIndex, 2, true, 0, 0);
			end;
			self.fixAttacher = false;
		end;

		if self:getIsActive()  then
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
				self.Go.animFoldExtend = not self.Go.animFoldExtend;
				self.Done.animFoldExtend= true;
			end;
		end;
	end;

	local joint = self.componentJoints[1];
	setJointFrame(joint.jointIndex, 0,joint.jointNode);
	local joint = self.componentJoints[2];
	setJointFrame(joint.jointIndex, 0,joint.jointNode);

end;

function Volokusha:draw()
	g_currentMission:addHelpButtonText(string.format(g_i18n:getText(self.vehicleType..".1"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
end;

function Volokusha:onAttach()
	self.fixAttacher = true;
end;

function Volokusha:onDetach()
 	local joint = self.saveJoint;
	joint.minRot = self.rotationCopy;
	joint.maxRotLimit = self.maxRotJoint;
end;

function Volokusha:anim(varName, loopCheck)	
	
	self.Activated[varName] = true;
	local loopCheck = self.loopCheck[varName];
	local speed = Utils.getNoNil(self.Speed[varName],1);
	 
	if self.Go[varName] == true and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, speed);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck == false then
			disableAnimTrack(charId, 0);	
			self.Done[varName] = false;	
			self.CheckDone[varName] = true;	
			self.Activated[varName] = false;
		
		end; 
	elseif self.Go[varName] == false and self.Done[varName] ~= false then
		local charId = getAnimCharacterSet(self.charId[varName]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
		if loopCheck == true then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
		end; 	
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackLoopState(charId, 0, loopCheck);
		setAnimTrackSpeedScale(charId, 0, -speed);
		enableAnimTrack(charId, 0);
		if getAnimTrackTime(charId, 0) <= 0 then
			disableAnimTrack(charId, 0);
			self.Done[varName] = false;	
			self.CheckDone[varName] = false;
			self.Activated[varName] = true;
		end;
	end;
end;
